@@ -325,3 +325,95 @@ class AlphaPCA(BaseEstimator, ClassifierMixin):
             u *= signs
             v *= signs.unsqueeze(-1)
         return u, v
+
+    def compute_optimal_alpha(self, X, alphas=None, n_components=None, degree=3, reconstruction_loss="mae"):
+        """Return the optimal alpha from data points.
+        Parameters
+        ----------
+        X : array-like of shape (n_samples, n_features)
+            The data.
+        alphas : a list of alpha values (default: [0.6, 0.8, 1.2, 1.4])
+        n_components : number of components, (default: self.n_components)
+        degree : polynomial degree fit (2 or 3, default: 3)
+        reconstruction_loss : reconstruction loss function in ['mae', 'mse', 'std'] (default: 'mae')
+        Returns
+        -------
+        best_alpha : float
+            Approximated optimal alpha.
+        """
+
+        from sklearn.preprocessing import PolynomialFeatures
+        from sklearn.linear_model import LinearRegression
+        import math 
+
+        def mae(X, X_, is_torch):
+            if is_torch:
+                X = X.cpu().numpy()
+                X_ = X_.cpu().numpy()
+            return np.abs(X - X_).mean()
+        
+        def mse(X, X_, is_torch):
+            if is_torch:
+                X = X.cpu().numpy()
+                X_ = X_.cpu().numpy()
+            return np.power(X - X_, 2).mean()
+        
+        def std(X, X_, is_torch):
+            if is_torch:
+                X = X.cpu().numpy()
+                X_ = X_.cpu().numpy()
+            return (X - X_).std()
+
+        if n_components is None:
+            n_components = self.n_components
+        if alphas is None:
+            if degree >= 3:
+                alphas = [0.6, 0.8, 1.2, 1.4]
+            elif degree == 2:
+                alphas = [0.6, 1.0, 1.4]
+
+        assert degree in [2, 3], "Degree must be 2 or 3"
+        assert isinstance(alphas, list), "Variable alphas must be a list"
+        assert len(alphas), "len(alphas) should be >= 3"
+        assert reconstruction_loss in ["mae", "mse", "std"], "reconstruction_loss must in ['mae', 'mse', 'std']"
+        
+        reconstruction_losses = []
+        for alpha in alphas:
+            pca = AlphaPCA(
+                n_components=n_components, 
+                copy=self.copy, 
+                center=self.center, 
+                precenter=self.precenter, 
+                prereduce=self.prereduce, 
+                do_inverse=self.do_inverse,
+                alpha=alpha,
+                n_oversamples=self.n_oversamples,
+                random_state=self.random_state
+                )
+            pca.fit(X)
+            is_torch = pca.is_torch
+
+            if reconstruction_loss == "mae":
+                reconstruction_losses.append(mae(X, pca.approximate(X), is_torch))
+            elif reconstruction_loss == "mse":
+                reconstruction_losses.append(mse(X, pca.approximate(X), is_torch))
+            elif reconstruction_loss == "std":
+                reconstruction_losses.append(std(X, pca.approximate(X), is_torch))
+
+        poly = PolynomialFeatures(degree=degree, include_bias=False)
+
+        #reshape data to work properly with sklearn
+        poly_features = poly.fit_transform(np.array(alphas).reshape(-1, 1))
+
+        #fit polynomial regression model
+        poly_reg_model = LinearRegression()
+        poly_reg_model.fit(poly_features, np.array(reconstruction_losses).reshape(-1, 1))
+
+        if degree == 2:
+            b, a = poly_reg_model.coef_[0]
+            return -b / (2*a)
+        elif degree == 3:
+            c, b, a = poly_reg_model.coef_[0]
+            delta = 4 * b**2 - 12 * a * c 
+            assert delta >= 0, "Failed to find the optimal alpha, consider using more data points"
+            return (-2*b + math.sqrt(delta)) / (6*a)
\ No newline at end of file
